import wx
import SimUtils

N = North = 0
E = East = 1
S = South = 2
W = West = 3
U = Up = 4
D = Down = 5

DirAbbrevArray = ("N", "E", "S", "W", "U", "D")
DirNameArray = ("NORTH", "EAST", "SOUTH", "WEST", "UP", "DOWN")


class Glue:
    """A representation of a glue on the side of a tile, including strength and label"""
    strength = 0
    label = ""
    hash = None

    def __init__(self, strength = 0, label = ""):
        self.strength = strength
        self.label = label

    def __eq__(self, other):
        if (isinstance(other, Glue)):
            return (self.strength == other.strength) and (self.label == other.label)
        return False

    def Copy(self):
        return Glue(self.strength, self.label)

    def Contains(self, string):
        return (self.label.find(string) > -1)

    def Print(self, d):
        print(DirNameArray[d] + "BIND " + str(self.strength))
        print(DirNameArray[d] + "LABEL " + self.label)

    def Output(self, d):
        ret  = DirNameArray[d] + "BIND " + str(self.strength) + "\n"
        ret += DirNameArray[d] + "LABEL " + self.label + "\n"
        return ret
    
    def Hash(self):
        if self.hash is None:
            self.hash = self.label + str(self.strength)
        return self.hash

#----------------------------------------------------------------------------

class TileType:
    def __init__(self, name, width=100, height=100, label=None):
        self.name = name
        self.label = label
        self.color = "White"
        self.colorVector = (255, 255, 255)
        self.width = width
        self.height = height
        self.dims = 2
        self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue()}
        self.fullString = None

    def DrawAtLocation(self, xPixel, yPixel, pdc):
        pdc.DrawRectangle(xPixel, yPixel, self.width, self.height)
        
    def Print(self):
        if self.IsEmpty():
            return
        if self.name == "":
            self.CreateName()
        print("TILENAME " + self.name)
        print("LABEL " + self.label)
        for i,d in self.sides.items():
            self.sides[i].Print(i)
        if len(self.color) > 0:
            print("TILECOLOR " + self.color)
        print("CREATE\n")

    def Output(self):
        if self.IsEmpty():
            return ""
        if self.name == "":
            self.CreateName()
        ret  = "TILENAME " + self.name + "\n"
        if not self.label is None:
            ret += "LABEL " + self.label + "\n"
        for i,d in self.sides.items():
            ret += self.sides[i].Output(i)
        if len(self.color) > 0:
            ret += "TILECOLOR " + self.color + "\n"
        ret += "CREATE\n"
        return ret
    
    def IsEmpty(self):
        for i in range(self.dims*2):
            if not self.sides[i].strength == 0:
                return False
        return True

    def GetName(self):
        if len(self.name) == 0:
            self.CreateName(self)
        return self.name

    def CreateName(self):
        for i in range(self.dims*2):
            self.name = self.name + DirAbbrevArray[i] + str(self.sides[i].strength) + self.sides[i].label

    def __str__(self):
        return self.GetName()
            
    def ToString(self, bFull=False):
        if bFull:
            if self.fullString is None:
                self.fullString = self.GetName() + " "
                for side in self.sides:
                    self.fullString += DirAbbrevArray[side] + ":" + self.sides[side].label + "(" + str(self.sides[side].strength) + ") "
                return self.fullString
            else:
                return self.fullString
        else:
            return self.GetName()

class Tile:
    def __init__(self, ttype, coords=None):
        self.type = ttype
        self.coords = coords

    def __str__(self):
        outStr = self.type.name + str(self.coords)
        return outStr

    def __repr__(self):
        outStr = self.type.name + str(self.coords)
        return outStr
